/*******************************************************************
*
*	LinuxCNC - Arduino-IO
*	ethraw - (c) by Ralf Burger 2019 - 
*	V.: 0.1 Last change:  22.02.2019 
*
* Description:
* This program was written for an Adruino-UNO with a W5100-Ethernet-shield.
* A direct ethernet-connection to the correspondig linuxCNC-eth-device
* is nessesarry.
*
* Author: Ralf Burger
* License: GPL Version 2
*    
* Copyright (c) 2019 All rights reserved.
*
********************************************************************
    This program is free software; you can redistribute it and/or
    modify it under the terms of version 2 of the GNU General
    Public License as published by the Free Software Foundation.
    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public
    License along with this library; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111 USA

    THE AUTHORS OF THIS LIBRARY ACCEPT ABSOLUTELY NO LIABILITY FOR
    ANY HARM OR LOSS RESULTING FROM ITS USE.  IT IS _EXTREMELY_ UNWISE
    TO RELY ON SOFTWARE ALONE FOR SAFETY.  Any machinery capable of
    harming persons must have provisions for completely removing power
    from all motors, etc, before persons enter any danger area.  All
    machinery must be designed to comply with local and national safety
    codes, and the authors of this software can not, and do not, take
    any responsibility for such compliance.
*********************************************************************

 

10 output-pins
8   input-pins


TESTING
=======
Configure the MAC-adresses "myadr" and "radr" to your needs.
Compile programm, upload to arduino and start it.

Start tcpdump on the connected PC:
sudo tcpdump -i eno1 -s 2000 ether host af:fe:af:fe:af:fe or ether host 70:85:c2:89:68:b7

Whenever you change an arduino-input-pin, you should see packets like this
in your tcpdump:

11:22:06.902447 af:fe:af:fe:af:fe (oui Unknown) > 70:85:c2:89:68:b7 .... Unknown) Null Information, length 50
	0x0000:  0000 003f 0000 0000 001b 1b21 0000 0000  ...?.......!....
	0x0010:  0000 0000 0000 0000 0000 0000 0000 0000  ................
	0x0020:  0000 0000 0000 0000 0000 0000 1b00 2100  ..............!.
	0x0030:  0000                                     ..


On the PC-side start 
sudo ./pcIOethRaw -i eno1 -s af:fe:af:fe:af:fe  -r 70:85:c2:89:68:b7 

press "o" (for output-mode) - no echo

press number-keys 1-8. 
each key toggles an output pin (A0 - A7) of the arduino. 

press "+".
you will get: "IO-byte:1"

press key "1" or "2".
they will toggle output-pins "A9" and "A10".



*/
 
#include <SPI.h>         // needed for Ethernet library communication with the W5100 (Arduino ver>0018)
#include <Ethernet.h>
#include <utility/w5100.h>

#define RECPKTCOUNT 58
#define SNDPKTCOUNT 60

static int sbuflen=64; // length of data to send

// the MAC-address of myself
static byte myadr[6]={0xAF,0xFE,0xAF,0xFE,0xAF,0XFE};

// the MAC-address of my the linuxCNC-Master
static byte radr [6]={0x70,0x85,0xc2,0x89,0x68,0xb7};
//static byte radr [6]={0xb8,0x27,0xeb,0xd6,0x96,0x81};
//static byte radr [6]={0xbc,0x5f,0xf4,0x7b,0xce,0x4d };

static int sendcount=0;   // packtes received 
static int reccount=0;    // packts sent
static int lastrec=0;     // last received packet-nr

static byte lastC;        // last state od imput-port C 
			  // (analog-input used as digital-input)

SOCKET s; 		  // socket that will be opened in RAW mode
static byte rbuf[1500+14];// receive buffer
static byte sbuf[1500+14];// send buffer
static int rbuflen; 	  // length of data to receive

// the setup runs once when you press reset
void setup() {                
 W5100.init();
 W5100.writeSnMR(s, SnMR::MACRAW);
 W5100.execCmdSn(s, Sock_OPEN);

	// first byte in IN packet
  pinMode(0, OUTPUT); // this is an output-pin
  pinMode(1, OUTPUT); // this is an output-pin
  pinMode(2, OUTPUT); // this is an output-pin
  pinMode(3, OUTPUT); // this is an output-pin
  pinMode(4, OUTPUT); // this is an output-pin
  pinMode(5, OUTPUT); // this is an output-pin
  pinMode(6, OUTPUT); // this is an output-pin
  pinMode(7, OUTPUT); // this is an output-pin

       // second byte in IN packet
  pinMode(8, OUTPUT); // this is an output-pin
  pinMode(9, OUTPUT); // this is an output-pin
//pinMode(10,OUTPUT); // this is an output-pin


      // first byte in OUT packet
  pinMode(A0, INPUT_PULLUP);  // set pull-up on analog pin 0 
  pinMode(A1, INPUT_PULLUP);  // set pull-up on analog pin 1 
  pinMode(A2, INPUT_PULLUP);  // set pull-up on analog pin 2 
  pinMode(A3, INPUT_PULLUP);  // set pull-up on analog pin 3 
  pinMode(A4, INPUT_PULLUP);  // set pull-up on analog pin 4 
  pinMode(A5, INPUT_PULLUP);  // set pull-up on analog pin 5 
  pinMode(A6, INPUT_PULLUP);  // set pull-up on analog pin 6 
  pinMode(A7, INPUT_PULLUP);  // set pull-up on analog pin 7 


  // put MAC-addresses into send-buffer
  for (int i=0; i<6; i++) sbuf[i]  =radr [i];
  for (int i=0; i<6; i++) sbuf[i+6]=myadr[i];
}


void send_pkt(){
  	memcpy(sbuf+SNDPKTCOUNT,&sendcount,2);
	// sbuf[20]=0x21;
 	// sending packet   // only  for DEBUGING
   	W5100.send_data_processing(s, sbuf, sbuflen);
   	W5100.execCmdSn(s, Sock_SEND_MAC);
        sendcount++;
	}


// the loop routine runs over and over again forever:
void loop() {
unsigned int     RxPacketSize;
 // check if we have received something
 rbuflen = W5100.getRXReceivedSize(s);
 if (rbuflen>0){
    W5100.recv_data_processing(s, rbuf, 2); // read the first two bytes
    W5100.execCmdSn(s, Sock_RECV);          // that tell us the lenght of the packet + 2
    RxPacketSize = rbuf[0];                 // hight byte
    RxPacketSize  <<= 8;                    //
    RxPacketSize  |=( rbuf[1] & 0x00FF);    // low byte
    RxPacketSize  -= 2;                     //  packet lenght
    W5100.recv_data_processing(s, rbuf, RxPacketSize);  // read the packet
    W5100.execCmdSn(s, Sock_RECV);

    // is this a packet for myself?
if (rbuf[0]==myadr[0])if (rbuf[1]==myadr[1])
if (rbuf[2]==myadr[2])if (rbuf[3]==myadr[3])
if (rbuf[4]==myadr[4])if (rbuf[5]==myadr[5])

if (rbuf[6] ==radr[0]) if (rbuf[7] ==radr[1])
if (rbuf[8] ==radr[2]) if (rbuf[9] ==radr[3])
if (rbuf[10]==radr[4]) if (rbuf[11]==radr[5])
{
	reccount++;    // increase number of sent packetes
        for (int i=0; i<6; i++) sbuf[i]  =radr [i];
        for (int i=0; i<6; i++) sbuf[i+6]=myadr[i];
    sbuf[23]=sendcount;
    sbuf[24]=sendcount;
    sbuf[25]=0x21;
    PORTD=rbuf[17];    // set the output-pins  (first 8 bit)
		       // now set the first 3 bits of next port
		       // (pin-nr.>10 ar used by ethernet-shield)
    digitalWrite(8, (rbuf[18]&1));  
    digitalWrite(9 ,(rbuf[18]&2));

    memcpy(sbuf+RECPKTCOUNT,&reccount,2);

         }  // packet was for me....
            // my work is done
} //sth. received

	// send acc if packet received
	if (lastrec!=reccount){ 
		send_pkt();
		lastrec=reccount;
		}
        // the pin-values of A0-A7 changed?
	if (lastC!=PINC){
                        sbuf[17]=PINC;
			lastC=PINC;
			send_pkt();
			}
			

}






